import numpy as np
import matplotlib.pyplot as plt
import datetime as dt
import pylab
from matplotlib.font_manager import FontProperties
import netCDF4 as nc4

font_large = FontProperties(size=18)

buoy_file = # Set input file: a netCDF4 file containing a dataset of monthly mean energy flux data from IMBs

outfile = # Set location of output file

letters = ['(a)','(b)','(c)','(d)']
variables = ['top_melt','fcondtop','fcondbot','ocean_heat_flux']
best_guess_salinities = {'top_melt':0,'fcondtop':1,'fcondbot':4,
                         'ocean_heat_flux':4}
titles = {'top_melt': 'Top melt flux','fcondtop':'Top conductive flux',\
        'fcondbot':'Basal conductive flux','ocean_heat_flux':'Ocean heat flux'}

region_names = ['North_Pole','Beaufort_Sea']
region_codes = {'North_Pole':1,'Beaufort_Sea':2}
region_colours = {'North_Pole':'b','Beaufort_Sea':'r'}
gridname = 'hadgom'
model_color = 'r'
nsample = 400

xx_plot = np.arange(.5,12,1.)
offset = .16
width = .15
figure = plt.figure(figsize=(12,12))

xl = .1
yb = .57
ys = .47
yw = .4
xw = .36
ygap = .48
xgap = .48

xx_plot = np.arange(.35,12,1)
offset = .3

ncid = nc4.Dataset(buoy_file)

for (ii,variable) in enumerate(variables):

    xi = ii % 2
    yi = ii // 2
    
    axis = figure.add_axes([xl+xgap*xi,yb-ygap*yi,xw,yw])
    
    axis.set_ylabel('Flux ($W/m^2$)',fontproperties=font_large)
    
    for (iregion,region_name) in enumerate(region_names):
        all_dists = []
        for month in range(1,13):
            index = np.where(np.logical_and(ncid.variables['month'][:]==month,
                             ncid.variables['region_code'][:]==region_codes[region_name]))
                       
            isalin = np.where(
               ncid.variables['salinity'][:]==best_guess_salinities[variable])[0]
            dist_2D = ncid.variables[variable][index[0],:]
            distribution = []
            for irec in range(dist_2D.shape[0]):
                item_array = dist_2D[irec,:]
                if item_array.mask[isalin] == False:
                    distribution.append(
                        item_array.data[isalin])
                elif False in item_array.mask:
                    distribution.append(
                        item_array.data[np.argmax(item_array.mask)-1])
                else:
                    pass

            distribution = np.array(distribution)
            all_dists.append(distribution)
      
        means = [np.mean(dist_nma) for dist_nma in all_dists]
        bp = axis.boxplot(all_dists,positions=xx_plot+offset*iregion,
                          widths = .2,whis=10)
        axis.plot(xx_plot+offset*iregion,means,linestyle=' ',marker='o',
            markerfacecolor=region_colours[region_name],
            markeredgecolor='k')

        for item in ['boxes','whiskers','fliers','means','medians','caps']:
            plt.setp(bp[item],color=region_colours[region_name])

    ylim = axis.get_ylim()
    for month in range(1,13):
        month_name = dt.date(1980,month,1).strftime('%b')
        axis.text(month-.5,ylim[0]-.06*(ylim[1]-ylim[0]),month_name[0],ha='center',\
            fontproperties = font_large)
        
        axis.plot([month-1,month-1],ylim,color='#c0c0c0')
    
    axis.set_xticks(np.arange(0,13))
    for label in axis.get_xticklabels():
        label.set_visible(False)
    
    for label in axis.get_yticklabels():
        label.set_fontproperties(font_large)
    
    axis.plot([0,12],[0,0],linestyle='--',color = 'k')
    axis.set_title(letters[ii],fontproperties=font_large)
    axis.set_ylim(ylim)

for region_name in region_names:
    axis.errorbar([0],[0],yerr=[0],label=region_name.replace('_',' '),\
        color=region_colours[region_name])

axis.legend(bbox_to_anchor=[-.3,-.56,.5,.5],ncol = 2)
pylab.savefig(outfile)
